'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import os
import sys
import data
import math
import random
import pygame

# game constants

NOF_ISLANDS = 20
ISLAND_SPEED = 0.5
ISLAND_WIDTH = 15

NOF_SPUMES = 250
SPUME_SPEED = 2.0
SPUME_WIDTH = 2.0

NOF_TIGERS = 2
TIGER_SPEED = 1.0
TIGER_WIDTH = 8.0

NOF_MAX_PEOPLE = 6

MAX_ROPE_DISTANCE = 100
NOF_ROPE_PEOPLE = 2
NOF_ANCHOR_PEOPLE = 5
CHECK_NOF_PEOPLE = True

NOF_RESCUE_PEOPLE = 20

DRIFTING_CHANCE = 0.1

ISLAND_MIGRATION_DISTANCE = ISLAND_WIDTH*2 + 5
TIGER_SCARE_DISTANCE = ISLAND_WIDTH + TIGER_WIDTH + 15

FRAMES_NEEDED_FOR_MIGRATION = 33 
FRAMES_NEEDED_FOR_SCARING = 33

class LevelData:
  def __init__( self ):
    self.islands = []
    self.tigers = []
    self.rescue_people = []
    self.nof_spumes = []
    
    # level 1
    # island tuple structure = pos, speed, nof_people
    self.islands.append( [] )
    self.islands[0].append( ((50,50)  ,ISLAND_SPEED,7) )
    self.islands[0].append( ((150,50) ,ISLAND_SPEED,4) )
    self.islands[0].append( ((50,10)  ,ISLAND_SPEED,9) )
    self.islands[0].append( ((250,250),ISLAND_SPEED,2) )
    self.rescue_people.append( 15 )
    # tiger tuple structure = pos, speed
    self.tigers.append( [] )
    self.tigers[0].append( ((300,300), TIGER_SPEED) )
    # spumes  
    self.nof_spumes.append( 250 )
    return

def normalize_vector( x, y ):
  length = vector_length( x, y )
  if length == 0.0:
    return (0.0,0.0)
  norm_x = x / length
  norm_y = y / length
  norm_x = max( norm_x, -1.0 )
  norm_y = max( norm_y, -1.0 )
  norm_x = min( norm_x, 1.0 )
  norm_y = min( norm_y, 1.0 )
  return (norm_x, norm_y)

def vector_length( x, y ):
  return math.sqrt(x*x + y*y)

class Particle:
  def __init__( self, map, pos_x=-1, pos_y=-1 ):
    self.map = map
    self.color = ( 255, 255, 0 )
    self.scale_speed = 1.0
    self.width = 10
    self.dir_x = 0.0
    self.dir_y = 0.0
    if pos_x==-1 or pos_y==-1:
      self.reset_random()
    else:
      self.pos_x = pos_x
      self.pos_y = pos_y
    return
  
  def handle_mouse_click( self, pos_x, pos_y ):
    if pos_x>=self.pos_x-self.width/2.0 and pos_x<=self.pos_x+self.width/2.0:
       if pos_y>=self.pos_y-self.width/2.0 and pos_y<=self.pos_y+self.width/2.0:
         return True
         #self.anchored = not self.anchored
         #self.color = ( 255-self.color[0], 255-self.color[1], 255-self.color[2] )
    return False
  
  def reset_random( self ):
    self.pos_x = random.randrange( self.map.pos_x, self.map.pos_x+self.map.width ) 
    self.pos_y = random.randrange( self.map.pos_y, self.map.pos_y+self.map.height )
    return
  
  def tick( self ):
    dir_x, dir_y = self.map.get_new_direction( self.pos_x, self.pos_y )

    dir_x = self.dir_x + dir_x
    dir_y = self.dir_y + dir_y

    # drift a little towards the center
    drifting = float(random.randrange(1000.0)) / 1000.0
    if drifting <= DRIFTING_CHANCE:
      center = ( self.map.pos_x+self.map.width/2, self.map.pos_y+self.map.height/2 )
      vector = ( self.pos_x-center[0], self.pos_y-center[1] )

      norm_x, norm_y = normalize_vector( vector[0], vector[1] )
      length = vector_length( vector[0], vector[1] )
      max_dist = vector_length( self.map.width/2, self.map.height/2 )
      factor = 1.0
      self.pos_x = center[0] + norm_x * length * factor
      self.pos_y = center[1] + norm_y * length * factor
    else:
      self.pos_x = self.pos_x + dir_x*self.scale_speed 
      self.pos_y = self.pos_y + dir_y*self.scale_speed

    self.dir_x = 0.0
    self.dir_y = 0.0

    reset_island = False
    if self.pos_x < self.map.pos_x:
      reset_island = True
    if self.pos_y < self.map.pos_y:
      reset_island = True
    if self.pos_x > self.map.pos_x+self.map.width:
      reset_island = True
    if self.pos_y > self.map.pos_y+self.map.height:
      reset_island = True
    if reset_island:
      self.reset_random()
    return
  
  def draw( self, screen ):
    pygame.draw.circle( screen, self.color, (int(self.pos_x),int(self.pos_y)), int(self.width), 0 )
    return

class Island( Particle ):
  def __init__( self, map, pos_x=-1, pos_y=-1  ):
    Particle.__init__(self, map, pos_x, pos_y)
    self.color = ( 255, 255, 0 )
    self.scale_speed = ISLAND_SPEED
    self.width = ISLAND_WIDTH
    self.people = random.randrange( NOF_MAX_PEOPLE )
    self.migration_progress = 10
    self.anchored = False
    self.flaged = False
    self.connected_with = []
    return
  
  def reset_random( self ):
    Particle.reset_random( self )
    self.people = random.randrange( NOF_MAX_PEOPLE )
    return
  
  def tick( self ):
    # movement
    if self.anchored == False:
      self.dir_x = 0.0
      self.dir_y = 0.0
      for island in self.connected_with:
        x = island.pos_x - self.pos_x
        y = island.pos_y - self.pos_y
        # forbid that connected islands pull them into each other
        if vector_length( x, y ) <= self.width+island.width:
          continue
        x, y = normalize_vector( x, y )
        self.dir_x = self.dir_x + x
        self.dir_y = self.dir_y + y
      Particle.tick( self )
    
    # migration
    for neighbour in self.map.islands:
      # dont handle myself
      if neighbour == self:
        continue
      # check if this other island is close enough to me, to do migration
      distance = vector_length( self.pos_x-neighbour.pos_x, self.pos_y-neighbour.pos_y )
      if distance <= ISLAND_MIGRATION_DISTANCE:
        
        if self.flaged==True and neighbour.flaged==True:
          if self.people>neighbour.people and self.people>0:
            self.migration_progress = self.migration_progress - 1
            if self.migration_progress<=0:
              self.people = self.people - 1
              neighbour.people = neighbour.people + 1
              self.migration_progress = FRAMES_NEEDED_FOR_MIGRATION
          else:
            if neighbour.people>self.people and neighbour.people>0:
              neighbour.migration_progress = neighbour.migration_progress - 1
              if neighbour.migration_progress<=0:
                self.people = self.people + 1
                neighbour.people = neighbour.people - 1
                neighbour.migration_progress = FRAMES_NEEDED_FOR_MIGRATION
        else:
          if self.flaged==True and neighbour.people>0:
            neighbour.migration_progress = neighbour.migration_progress - 1
            if neighbour.migration_progress <= 0 :
              self.people = self.people + 1
              neighbour.people = neighbour.people - 1
              neighbour.migration_progress = FRAMES_NEEDED_FOR_MIGRATION
          if neighbour.flaged==True and self.people>0:
            self.migration_progress = self.migration_progress - 1
            if self.migration_progress <= 0:
              self.people = self.people - 1
              neighbour.people = neighbour.people + 1
              self.migration_progress = FRAMES_NEEDED_FOR_MIGRATION
    return
  
  def draw( self, screen ):
    Particle.draw( self, screen )
    
    # draw people
    if self.people > 0:
      self.map.game.draw_text( str(self.people), self.pos_x-self.width/3*2, self.pos_y-self.width, (0,0,0) )
    
    # draw anchor
    if self.anchored:
      self.map.game.draw_text( 'A', self.pos_x, self.pos_y-self.width/3*2, (0,0,0) )
    
    # draw flag
    if self.flaged:
      self.map.game.draw_text( 'F', self.pos_x-self.width/3*2, self.pos_y-self.width/4.0, (0,0,0) )
    
    # draw rope
    for island in self.connected_with:
      start = ( self.pos_x, self.pos_y )
      end   = ( island.pos_x, island.pos_y )
      pygame.draw.line( screen, (0,0,0), start, end, 1 )
    return
    
  def handle_anchor_click( self ):
    ok = False
    if self.anchored == False:
      if CHECK_NOF_PEOPLE:
        if self.people >= NOF_ANCHOR_PEOPLE:
          ok = True
      else:
        ok = True
      
      if ok == True:
        self.anchored = True
        self.map.game.status_string = "the clicked islands sets anchor"
        if CHECK_NOF_PEOPLE:
          self.people = self.people - NOF_ANCHOR_PEOPLE
    else:
      self.anchored = False
      if CHECK_NOF_PEOPLE:
        self.people = self.people + NOF_ANCHOR_PEOPLE
      self.map.game.status_string = "the clicked islands released the anchor"
    return
  
  def handle_rope_click( self ):
    if self.connected_with != []:
      for island in self.connected_with:
        island.connected_with = []
      self.connected_with = []
      self.map.game.rope_island = None
      return
    
    # only allow 1-1 ropes
    if self.map.game.rope_island == None:
      if CHECK_NOF_PEOPLE:
        if self.people < NOF_ROPE_PEOPLE:
          return  
        self.map.game.rope_island = self
    else:
      if self.map.game.rope_island == self:
        self.map.game.rope_island = None
        return
      # check distance between them
      other_x = self.map.game.rope_island.pos_x
      other_y = self.map.game.rope_island.pos_y

      if vector_length(self.pos_x-other_x, self.pos_y-other_y) <= MAX_ROPE_DISTANCE:
        self.map.game.rope_island.connected_with.append( self )
        self.connected_with.append( self.map.game.rope_island )
        if CHECK_NOF_PEOPLE:
          self.map.game.rope_island.people = self.map.game.rope_island.people - NOF_ROPE_PEOPLE 
        self.map.game.rope_island = None
    return
  
  def handle_flag_click( self ):
    self.flaged = not self.flaged
    return
  
  def handle_mouse_click( self, pos_x, pos_y ):
    if Particle.handle_mouse_click( self, pos_x, pos_y ):
      # anchor mode
      if self.map.game.mouse_mode == self.map.game.mouse_modes[1]:
        self.handle_anchor_click()
      # rope mode
      if self.map.game.mouse_mode == self.map.game.mouse_modes[2]:
        self.handle_rope_click()
      # rope mode
      if self.map.game.mouse_mode == self.map.game.mouse_modes[3]:
        self.handle_flag_click()
      return

class Spume( Particle ):
  def __init__( self, map, pos_x=-1, pos_y=-1  ):
    Particle.__init__(self, map, pos_x, pos_y)
    self.color = ( 255, 255, 255 )
    self.scale_speed = SPUME_SPEED
    self.width = SPUME_WIDTH
    return
  
class Tiger( Particle ):
  def __init__( self, map, pos_x=-1, pos_y=-1  ):
    Particle.__init__(self, map, pos_x, pos_y)
    self.color = ( 0, 0, 0 )
    self.scale_speed = TIGER_SPEED
    self.width = TIGER_WIDTH
    self.scaring_progress = FRAMES_NEEDED_FOR_SCARING
    return
  
  def tick( self ):
    Particle.tick( self )
    # check if tiger will scare some people
    scared_someone = False
    for island in self.map.islands:
      distance = vector_length( self.pos_x-island.pos_x, self.pos_y-island.pos_y )
      if distance <= TIGER_SCARE_DISTANCE:
        if island.people > 0:
          scared_someone = True
          self.scaring_progress = self.scaring_progress - 1
          if self.scaring_progress <= 0: 
            island.people = island.people - 1
            self.scaring_progress = FRAMES_NEEDED_FOR_SCARING
    if scared_someone == False:
      self.scaring_progress = FRAMES_NEEDED_FOR_SCARING
    return
  
  def draw( self, screen ):
    if self.scaring_progress == FRAMES_NEEDED_FOR_SCARING:
      self.color = ( 0, 0, 0 )
    else:
      self.color = ( 255, 0, 0 )
    Particle.draw( self, screen )
    # draw scaring range
    pygame.draw.circle( screen, self.color, (int(self.pos_x),int(self.pos_y)), int(TIGER_SCARE_DISTANCE-ISLAND_WIDTH), 1 )
    return
    

class Map:
  def __init__( self, game, pos_x, pos_y, width, height ):
    self.game = game
    self.pos_x = pos_x
    self.pos_y = pos_y
    self.width = width
    self.height = height
    self.scale_arrow = 10.0
    self.nof_arrows = 41
    self.nof_islands = NOF_ISLANDS
    self.islands = []
    self.nof_spumes = NOF_SPUMES
    self.spumes = []
    self.nof_tigers = NOF_TIGERS
    self.tigers = []
    self.rescue_people = 0
    self.win = False
    self.border_color = ( 255, 255, 255 )
    return
  
  def init_new_level( self, level ):
    self.win = False
    data = LevelData()
    
    self.rescue_people = NOF_RESCUE_PEOPLE
    self.islands = []
    for i in range( self.nof_islands ):
      self.islands.append( Island(self) )
    self.spumes = []
    for i in range( self.nof_spumes ):
      self.spumes.append( Spume(self) )
    self.tigers = []
    for i in range( self.nof_tigers ):
      self.tigers.append( Tiger(self) )
    
    #self.rescue_people = data.rescue_people[level]
    #c = 0
    #self.islands = []
    #for i in data.islands[level]:
    #  self.islands.append( Island(self) )
    #  self.islands[c].pos_x  = i[0][0]
    #  self.islands[c].pos_y  = i[0][1]
    #  self.islands[c].speed  = i[1]
    #  self.islands[c].people = i[2]
    #  c = c + 1

    #self.spumes = []
    #for i in range( data.nof_spumes[level] ):
    #  self.spumes.append( Spume(self) )
    
    #c = 0
    #self.tigers = []
    #for i in data.tigers[level]:
    #  self.tigers.append( Tiger(self) )
    # self.tigers[c].pos_x = i[0][0]
    #  self.tigers[c].pos_y = i[0][1]
    #  self.tigers[c].speed = i[1]
    #  c = c + 1
    return
  
  def handle_mouse_click( self, pos_x, pos_y ):
    for island in self.islands:
      if island.handle_mouse_click( pos_x, pos_y ):
        break
    for spume in self.spumes:
      spume.handle_mouse_click( pos_x, pos_y )
  
  def update_objects( self ):
    for island in self.islands:    
      island.tick()
    for spume in self.spumes:    
      spume.tick()
    for tiger in self.tigers:
      tiger.tick()

    return
  
  def get_new_direction( self, pos_x, pos_y ):
    pos_y = pos_y
    # map the coordinates to [0..1]
    x_scaled = float(pos_x) / float(self.width)
    y_scaled = float(pos_y) / float(self.height)
    
    # scale the vector to [-1.0, +1.0]
    x_scaled = (x_scaled * 2.0) - 1.0
    y_scaled = (y_scaled * -2.0) + 1.0

    # norm the vector
    x_scaled, y_scaled = normalize_vector( x_scaled, y_scaled )
    
    # determine alpha
    alpha = math.asin( float(x_scaled) )

    # rotate alpha
    rotate = math.pi*2.0/float(self.width)
    alpha = alpha + rotate
    
    # get new direction
    dir_x = math.cos( float(alpha) )
    dir_y = math.sin( float(alpha) )
    if y_scaled < 0.0:
      dir_x = dir_x * -1.0

    return (dir_x,dir_y)
  
  def drawArrow( self, screen, pos_x, pos_y ):
    points = []
    dir_x, dir_y = self.get_new_direction(pos_x, pos_y)
    
    pos_x = pos_x+self.pos_x
    pos_y = pos_y+self.pos_y
    start = (pos_x,pos_y)
    end = (pos_x+dir_x*self.scale_arrow,pos_y+dir_y*self.scale_arrow)
    
    pygame.draw.line( screen, self.border_color, start, end, 1 )
    
    return
  
  def drawArrows( self, screen ):
    nof_arrows = self.nof_arrows-1
    for y in range( self.nof_arrows ):
      for x in range( self.nof_arrows ):
        pos_x = x * self.width/nof_arrows
        pos_y = y * self.height/nof_arrows
        #print "x=[0, %d, %d]->%d, y=[0, %d, %d]->%d" % (x, self.nof_arrows, pos_x, y, self.nof_arrows, pos_y)
        self.drawArrow( screen, pos_x, pos_y )
    return
  
  def draw( self, screen ):
    points = []
    points.append( (self.pos_x,self.pos_y) )
    points.append( (self.pos_x,self.pos_y+self.height) )
    points.append( (self.pos_x+self.width,self.pos_y+self.height) )
    points.append( (self.pos_x+self.width,self.pos_y) )
    
    # water surface
    pygame.draw.polygon( screen, (0,0,255), points, 0 )
    
    # border
    pygame.draw.lines( screen, self.border_color, True, points, 1 )
    
    # draw Arrows
    #self.drawArrows( screen )
    
    # spumes
    for spume in self.spumes:
      spume.draw( screen )
    
    # islands
    for island in self.islands:
      island.draw( screen )
      
    # tigers
    for tiger in self.tigers:
      tiger.draw( screen )
    
    # draw a circle around the selected rope island
    isle = self.game.rope_island
    if isle != None:
      pygame.draw.circle( screen, (0,0,0), (int(isle.pos_x),int(isle.pos_y)), isle.width, 3 )
      pygame.draw.circle( screen, (0,0,0), (int(isle.pos_x),int(isle.pos_y)), MAX_ROPE_DISTANCE-ISLAND_WIDTH, 1 )
    
    return
  
  def check_win_condition( self ):
    nof_peoples_on_a_single_island = 0
    for island in self.islands:
      nof_peoples_on_a_single_island = max( nof_peoples_on_a_single_island, island.people )
    if nof_peoples_on_a_single_island >= self.rescue_people:
      self.win = True
    return

class Button:
  def __init__( self, function, key, gfx, pos_x, pos_y ):
    self.function = function
    self.gfx = gfx
    self.key = key
    self.width = gfx[key].get_width()
    self.height = gfx[key].get_height()
    self.pos_x = pos_x
    self.pos_y = pos_y
    self.clicked = 0
    return
  
  def wasClicked( self, pos_x, pos_y ):
    if pos_x >= self.pos_x and pos_x <= self.pos_x+self.width and pos_y >= self.pos_y and pos_y <= self.pos_y+self.height:
      self.function()
      self.clicked = 10
      return True
    else:
      return False
  
  def draw( self, screen ):
    screen.blit( self.gfx[self.key], (self.pos_x,self.pos_y) )
    if self.clicked > 0:
      self.clicked = self.clicked -1
      dx = self.gfx[self.key].get_width()
      dy = self.gfx[self.key].get_height()
      x = self.pos_x
      y = self.pos_y
      points = [ (x,y), (x+dx,y), (x+dx,y+dy), (x,y+dy) ]
      pygame.draw.lines( screen, (255, 255, 255), True, points, 2 )
  

class Resources:
  def __init__( self ):
    self.gfx = {}
    self.load_image( "welcome", "welcome_screen.png" )
    self.load_image( "winning", "winning.png" )
    self.load_image( "help", "help_screen.png", 800, 600 )
    self.load_image( "anchor_button", "anchor_button.png", 64, 64 )
    self.load_image( "rope_button", "rope_button.png", 64, 64 )
    self.load_image( "flag_button", "flag_button.png", 64, 64 )
    self.load_image( "help_button", "help_button.png", 64, 64 )
    self.load_image( "back_button", "back_button.png", 64, 64 )
    self.load_image( "level_1_button", "level_1_button.png", 64, 64 )
    self.load_image( "quit_button", "quit_button.png", 64, 64 )
    return
  
  def load_image( self, key, file, new_width=0, new_height=0 ):
    # create filename
    filename = os.path.join( 'data', file )
    try:
        surface = pygame.image.load( filename )
    except pygame.error:
        raise SystemExit, 'Could not load image "%s" %s'%( filename, pygame.get_error())
    
    self.gfx[key] = surface.convert_alpha()
    if new_width>0 and new_height>0:
      self.gfx[key] = pygame.transform.scale( self.gfx[key], (new_width,new_height) )
    return


class Game:
  def __init__( self ):
    self.run_game = True
    # init the pygame stuff
    pygame.init()
    pygame.display.init()
    self.screen = pygame.display.set_mode( (800,600) )
    pygame.display.set_caption( "Batwima Islands v0.2" )
    self.font = pygame.font.Font( None, 20 )
    # init my stuff
    self.status_string = 'Welcome to Batwima Islands!'
    self.resources = Resources()
    self.map = Map( self, 10, 10, 780, 500 )
    self.mouse_modes = [ 'Default', 'Anchor', 'Rope', 'Flag' ]
    self.mouse_mode = 'Default'
    self.rope_island = None
    self.buttons = {}
    self.show_main_menu()
    return

  def quit_game( self ):
    self.run_game = False
    return

  def running( self ):
    return self.run_game

  def update_objects( self ):
    if self.game_mode == 'playing':
      if self.map.win == False:
        self.map.update_objects()
        self.map.check_win_condition()
    return
  
  def show_main_menu( self ):
    self.game_mode = 'main_menu'
    self.buttons = {}
    self.buttons['play_1'] = Button( self.start_level_1 ,"level_1_button",self.resources.gfx,50,523)
    self.buttons['help']   = Button( self.show_help_screen ,"help_button",self.resources.gfx,600,523)
    self.buttons['quit']   = Button( self.quit_game ,"quit_button",self.resources.gfx,700,523)
    return
  
  def init_game_buttons( self ):
    self.buttons = {}
    self.buttons['flag']   = Button( self.switch_to_flag_mode ,"flag_button",self.resources.gfx,300,523)
    self.buttons['anchor'] = Button( self.switch_to_anchor_mode ,"anchor_button",self.resources.gfx,400,523)
    self.buttons['rope']   = Button( self.switch_to_rope_mode ,"rope_button",self.resources.gfx,500,523)
    self.buttons['help']   = Button( self.show_help_screen ,"help_button",self.resources.gfx,600,523)
    self.buttons['quit']   = Button( self.show_main_menu ,"quit_button",self.resources.gfx,700,523)
    return
  
  def start_level_1( self ):
    self.game_mode = 'playing'
    self.init_game_buttons()
    self.map.init_new_level( 0 )
    return
  
  def show_help_screen( self ):
    self.last_game_mode = self.game_mode
    self.game_mode = 'help'
    self.buttons = {}
    self.buttons['help']   = Button( self.back ,"back_button",self.resources.gfx,700,523)
    return
  
  def back( self ):
    if self.last_game_mode == 'main_menu':
      self.show_main_menu()
    if self.last_game_mode == 'playing':
      self.game_mode = 'playing'
      self.init_game_buttons()
    return
  
  def switch_to_default_mode( self ):
    self.mouse_mode = self.mouse_modes[0]
    self.rope_island = None
    self.status_string = "switched to mouse mode " + self.mouse_mode
    return

  def switch_to_anchor_mode( self ):
    self.mouse_mode = self.mouse_modes[1]
    self.rope_island = None
    self.status_string = "switched to mouse mode " + self.mouse_mode
    return
  
  def switch_to_rope_mode( self ):
    self.mouse_mode = self.mouse_modes[2]
    self.status_string = "switched to mouse mode " + self.mouse_mode
    return
  
  def switch_to_flag_mode( self ):
    self.mouse_mode = self.mouse_modes[3]
    self.rope_island = None
    self.status_string = "switched to mouse mode " + self.mouse_mode
    return
  
  def handle_event( self, event ):
    if event.type == pygame.KEYDOWN:
      self.handle_keyboard_event( event )
    if event.type == pygame.MOUSEBUTTONUP:
      self.handle_mouse_event( event )
    return
  
  def handle_mouse_event( self, event ):
    (x,y) = pygame.mouse.get_pos()
    # handle buttons
    for key in self.buttons:
      if self.buttons[key].wasClicked( x, y ):
        return
    #handle map
    if self.game_mode == 'playing':
      self.map.handle_mouse_click( x, y )
    return
  
  def handle_keyboard_event( self, event ):
    if event.key == pygame.K_ESCAPE:
       self.run_game = False
    if event.key == pygame.K_F5:
      for island in self.map.islands:
        island.reset_random()
    return

  def draw_text( self, text, pos_x, pos_y, color ):
    rendered_text = self.font.render( text, 1, color )
    text_pos = rendered_text.get_rect()
    text_pos.topleft = ( pos_x, pos_y )
    self.screen.blit( rendered_text, text_pos )
    return
  
  def draw( self ):
    self.screen.fill( (0, 0, 0) )
    
    if self.game_mode == 'main_menu':
      self.screen.blit( self.resources.gfx['welcome'], (0,0) )

    if self.game_mode == 'help':
      self.screen.blit( self.resources.gfx['help'], (0,0) )
    
    if self.game_mode == 'playing':
      if self.map.win == False:
        self.map.draw( self.screen )
        self.draw_text( "Gather %d people on an island to win this level" % (self.map.rescue_people) , 10, 520, (255, 255, 255) )
        self.draw_text( "Press <F5> to start a new level" , 10, 550, (255, 255, 255) )
        self.draw_text( self.status_string , 10, 580, (255, 255, 255) )
      else:
        self.screen.blit( self.resources.gfx['winning'], (self.map.pos_x,self.map.pos_y) )
      
    
    for key in self.buttons:
      self.buttons[key].draw( self.screen )
    
    pygame.display.flip()
    return

def main():
    game = Game()
    
    while game.running():
      # event handling
      for event in pygame.event.get():
        if event.type == pygame.QUIT:
          game.quit_game()
        game.handle_event( event )
      
      # update objects
      game.update_objects()
      
      # draw new frame
      game.draw()
      
      # wait a little to get 30 fps
      pygame.time.delay( 33 )
      #game.quit()
      
    #end of game
    pygame.quit()
    sys.exit()
